function [CIH CIS CIF countb gcoxboots]=HCNSboots(x, status, defineevent,  Z, knots, kgon, options, CIat, boots, clevel, discard, profil)

%---Derives 95% bootstrap based confidence intervals for the HCNS estimate.
% INPUT ARGUMENTS: (The first 7 input arguments are obligatory)
%x     :    time to event variable that may be subject to right censoring
%status:    A Boolean array of the same size as x that is 1 for observations
%           that are right-censored and 0 for observations that are observed 
%           exactly.
%Z     :    A matrix that contains all covariates in its columns. The number
%           of its rows should be equal to the size of x. If no covariates are
%           available set as []. In the presence of covariates, a cox model is fitted
%           and smoothing is applied over the baseline cumulative step hazard.
%knots :    The knots of the spline that may be supplied by the user. Alternatively,
%           it can be set to 'auto' and an automatic choice between some knot
%           placement schemes will take over. The selection of the most 
%           appropriate scheme is based on the distance (sum of squares) of the Kaplan Meier's  
%           cumulative step hazard corners (occurring at event times), from the spline model.
%kgon :     A positive integer(>0). An optimal polygon with (8*kgon+2) corners, is used
%           to approximate the region of monotonicity. Check the references. If you
%           WE STRONGLY RECOMMEND NOT TO USE VERY LARGE VALUES:
%           Computational time will increase dramatically, and probably no
%           better results will be obtained than using k=2 or 3. 
%options:   As defined in the built in MATLAB routine coxphfit. Refers only to the 
%           cox model fitting procedure, dealing with maximum number of iterations 
%           as well as the toleration of convergence. See the coxphfit documentation.
%CIat:      an array of values at which the confidence intervals are to be derived.
%boots:     Number of bootstrap samples to be used.
%level:     The level of significance, set it equal to 0.05 to obtain 95%
%            confidence intervals for example.
%discard:   A positive number declaring the number of standard deviations based 
%           on which the intervals  gcoxhat+-discard*SE(gcoxhat) will be 
%           build. If for a bootstrap sample one or more of the cox coefficients lie 
%           outside the corresponding interval(s), then the bootstrap sample will be 
%           discarded and another one will be drawn instead. This is since all data 
%           might turn out to be censored or the last event for one group may be earlier 
%           than the first event of another for two categories of one covariate. 
%           In these cases one might end up trying to maximize a monotone likelihood 
%           during fitting a Cox model.
%
%
%OPTIONAL INPUT ARGUMENT
%profil:    An array of length equal to the number of columns of the covariate
%           matrix Z. It may contain a specific profile of covariate values and the
%           estimates produced will refer to that specific profile. If it is
%           set to 0, then estimates for the baseline are obtained (the same thing is done 
%           if this input argument in omitted). If this optional argument
%           is provided by the user the the CI's are obtained for this
%           profile of covariate values.


%OUTPUT:
%CIH       : Three column matrix. 1st column: the time at which the user
%            requested the desired confidence intervals. 2nd column: Lower value of the
%            confidence interval for the cumulative hazard. 3rd: Upper value of the confidence interval of the cumulative hazard.
%CIS       : Three column matrix. 1st column: the time at which the user
%            requested the desired confidence intervals. 2nd column: Lower value of the
%            confidence interval for the survival function. 3rd: Upper value of the confidence interval of the survival function.
%CIF       : Three column matrix. 1st column: the time at which the user
%            requested the desired confidence intervals. 2nd column: Lower value of the
%            confidence interval for the cumulative distribution. 3rd: Upper value of the confidence interval of the cumulative distribution.
%countb    : the number of discarded bootstrap samples until the desired
%            number of bootstrap samples is gathered.
%gcoxboots : a matrix with rows equal to the number of bootstrap samples
%            and columns equal to the number of covariates (i.e. equal to the number of columns in Z)






% For examples please see the examples.m file 

%Last updated: (25th January 2012).
%Code Author: Leonidas E. Bantis, 
%University of the Aegean, Dept. of Statistics and Actuarial-Financial Mathematics, Samos Island.

%e-mail: lbantis@aegean.gr, leobantis@gmail.com.
%An updated version of this software will be available online:

%-------www.leobantis.net23.net------------------------------


%References:
%Leonidas E. Bantis, John V. Tsimikas and Stelios D. Georgiou
%(2012). A MATLAB routine for Survival Estimation through the Cumulative Hazard Function with
%Monotone Natural Cubic Splines. Submitted for publication.

%Leonidas E. Bantis, John V. Tsimikas and Stelios D. Georgiou
%(2012). Survival Estimation through the Cumulative Hazard Function with
%Monotone Natural Cubic Splines. (Lifetime Data Analysis (To appear). Springer.)



%---Begin Error Checking:
if nargin<10;error('The function ''HCNSboots'' requires at least 8 input arguments');end
if nargin>12;error('Too many input arguments in function ''HCNSboots'' ');end
if isnumeric(x)~=1;error('Time variable must be numeric.');end
if max(status)>1;error('status variable must be a boolean (logical) vector.');end
if max(status)<0;error('status variable must be a boolean (logical) vector.');end
   
sizex=size(x);
if sizex(1)>1 && sizex(2)>1; error('Input argument ''x'' must be a row or a column vector');end
if sizex(1)==1;x=x';end

statusx=size(status);
if statusx(1)>1 && statusx(2)>1; error('Input argument ''status'' must be a row or a column vector');end
if statusx(1)==1;status=status';end


if sum(size(defineevent))~=2;error('The ''definevent'' value must be a scalar stating the value that the status variable takes in the case of an event  ');end
if defineevent~=1 && defineevent~=0;error('The ''defineevent'' must be either ''1'' or ''0'', depending on the coding selected for an event ');end
if defineevent==1;status=status.*(-1)+1;end


if length(find(status==1))==length(status);error('No analysis can be performed if all data are censored');end

if isnumeric(Z)~=1;error('Not valid argument for covariate matrix. If there are no covariates set the covariate input argument as [].');end

if isempty(Z)~=1
    sizeZ=size(Z);
    if sizeZ(1)~=length(x);error('The dimensions of the covariate matrix ''Z'' does not match the dimension of the time to event variable ''x''  ');end
end


if floor(kgon)~=ceil(kgon);error('The parameter ''k'' must be a positive integer greater than or equal to 1');end
if kgon<0;error('The parameter ''k'' must be a positive integer greater than or equal to 1');end


if isnumeric(knots)==1
    sizekn=size(knots);
    if sizekn(1)>1 && sizekn(2)>1;error('Knots must be an array and not a matrix');end
    if sizekn(1)>1;knots=knots';end
end


    if isempty(boots)==1;error('Not valid input argument for ''boots''  ');end
    if isnumeric(boots)~=1;error('Input argument for ''boots'' must be numeric ');end
    if sum(size(boots))~=2;error('Input argument ''boots'' must not be an array or matrix');end
    if floor(boots)~=ceil(boots);error('The parameter ''boots'' must be a positive integer. It is the number of the bootstrap samples that will be used.');end
    if boots<0;error('The parameter ''boots'' must be a positive integer. It is the number of the bootstrap samples that will be used.');end


if sum(size(clevel))~=2;error('The ''clevel'' value must be a scalar stating the level of significance to obtain the corresponding confidence intervals ');end
if clevel==1;error('The ''clevel'' value must be a scalar smaller than 1  ');end
if clevel==0;error('The ''clevel'' value must be a scalar greater than 0  ');end
    
%if nargin<11;discard=Inf; end;
if isempty(discard)==1;   discard=Inf; end;
if isnumeric(discard)~=1;error('Input argument for ''discard'' must be numeric ');end
if sum(size(discard))~=2;error('Input argument ''discard'' must not be an array or matrix');end   
if discard<=0;error('Input argument ''discard'' must be positive ');end


if nargin==12
    if isempty(profil)==1;error('Not valid input argument for ''profil''  ');end
    if isnumeric(profil)~=1;error('Input argument for ''profil'' must be numeric ');end
    sizeprof=size(profil);
    if sizeprof(1)>1 && sizeprof(2)>1;error('Input argument for ''profil'' must be an array and not a matrix');end
    
    sizeprof=size(profil); if sizeprof(2)>1;profil=profil';end    
    if length(profil)~=sizeZ(2);('The length of the profile array ''profil'' cannot exceed the number of the covariate matrix Z');end
end

sizeCIat=size(CIat);
if sizeCIat(1)~=1 && sizeCIat(2)~=1;error('The values at which the CIs are to be computed must a vector and not a matrix');end
if sizeCIat(1)==1;CIat=CIat';end


if nargin<12;profil=zeros(1, length(Z(1,:)));end %if the profil is not provided the take it to be all zero.

%---------------Done with error checking----------------------------





%-----Now deal with the discard option: build the desired interval---
%----outside which, a bootstrap sample will be discarded ------------

%First check the Cox model for the original data:
if isempty(Z)~=1
    if isempty(options)==1
        [gcox0,~,~,stats] = coxphfit(Z,x,'censoring', status,'baseline',0);
    else
        [gcox0,~,~,stats] = coxphfit(Z,x,'censoring', status,'baseline',0,'options',options);
    end
    
      stdsg=diag(stats.covb);stdsg=sqrt(stdsg);
      inters=[gcox0-discard*stdsg  gcox0+discard*stdsg];
      lowinters=inters(:,1);uppinters=inters(:,2);
      
    
    
    
end






%----End of dealing with the discard option.



n=length(x);
Ssam=zeros(boots,length(CIat)); %preallocate
Hsam=Ssam; %preallocate
Fsam=Ssam; %preallocate





if isempty(Z)==1   
       
       gcoxboots=NaN;
       i=0;countb=0;kkk=0;
       while i<=boots
        %for i=1:boots    
            if i==round(0.25*boots) && kkk==0;disp('About 25 percent of calculations are done...');kkk=kkk+1;end
            if i==round(0.50*boots) && kkk==1;disp('About 50 percent of calculations are done...');kkk=kkk+1;end
            if i==round(0.75*boots) && kkk==2;disp('About 75 percent of calculations are done...');kkk=kkk+1;end
           
            sam = randsample(n,n,true);
            %[bhat Hx Fx Sx KMdist knots gcoxhat stats]=HCNS(x, status, Z, knots, kgon, options, plots, profil, plotsprofil)
            
            if sum(status(sam))~=length(status);
            
                    if isempty(options)==1
                        [~, Hs, Fs, Ss ]=HCNS(x(sam), status(sam), 0, [], knots ,kgon);
                    else
                        [~, Hs, Fs, Ss ]=HCNS(x(sam), status(sam), 0, [], knots ,kgon, options);
                    end
                    

            Ssam(i,:)=Ss(CIat);
            Hsam(i,:)=Hs(CIat);
            Fsam(i,:)=Fs(CIat);
            i=i+1;
            
            else
                countb=countb+1; %the "i" remains the same, so this iteration will be replaced by another
                
                
            end
                
                
                
            
            
        
       end  %end of bootstrap when no covariates are present.
        
else 
       sizez=size(Z);colsz=sizez(2);
       Zsam=Z; %preallocate;
       
       countb=0; %start counting how many bootstrap samples will be discarded.
       j=1;kkk=0;
       gcoxboots=zeros(boots,colsz);
       while j<=boots
       %for j=1:boots
            if j==round(0.25*boots) && kkk==0;disp('About 25 percent of calculations are done...'); kkk=kkk+1;end
            if j==round(0.50*boots) && kkk==1;disp('About 50 percent of calculations are done...');kkk=kkk+1;end
            if j==round(0.75*boots) && kkk==2;disp('About 75 percent of calculations are done...');kkk=kkk+1;end
            

            sam = randsample(n,n,true);
       
                for i=1:colsz
                        zsam=Z(:,i);
                        zsam=zsam(sam);
                        Zsam(:,i)=zsam;
                end
            
            [~, Hs, Fs, Ss , ~,  ~, gcox]=HCNS(x(sam), status(sam), 0,  Zsam, knots ,kgon, options, 'none', profil);
            
            gcoxboots(j,:)=gcox';
            Ssam(j,:)=Ss(CIat);
            Hsam(j,:)=Hs(CIat);
            Fsam(j,:)=Fs(CIat);
            
            % Check if for the fitted cox within this bootstrap sample
            % the coefs are outside the intervals specified by the user
            % If at least one is outside then discard this bootstrap sample
            mustthrow=zeros(length(gcox0),1);%preallocate
            for gg=1:length(gcox);
                if gcox(gg)<lowinters(gg) || uppinters(gg)<gcox(gg)
                    mustthrow(gg)=1;
                end
            end
            mustthrow=sum(mustthrow);
            
            if mustthrow==0;j=j+1;end
            if mustthrow==1;countb=countb+1;end


            % Done with discarding the bootstraps that must be discarded.
            
            
       end
end

Hcis=zeros(length(CIat),2); %preallocate
Scis=Hcis; %preallocate
Fcis=Hcis; %preallocate

for i=1:length(CIat); 
    Hcis(i,:)=prctile(Hsam(:,i),[clevel/2*100  100-(clevel/2*100)]);
    Scis(i,:)=prctile(Ssam(:,i),[clevel/2*100  100-(clevel/2*100)]);
    Fcis(i,:)=prctile(Fsam(:,i),[clevel/2*100  100-(clevel/2*100)]);
end


CIH=[CIat Hcis];
CIS=[CIat Scis];
CIF=[CIat Fcis];
disp('All calculations are done.')

end
%------End of deriving CI's in the case of no covariates------------------

