function [bhat Hx Fx Sx KMdist knots gcoxhat stats]=HCNSd(x, status, defineevent, Z, knots, kgon, options, plots, profil, plotsprofil)

% Estimates the cumulative hazard function by smoothing over the Kaplan
% Meier based cumulative hazard step function using a spline based technique
% presented by Leonidas E. Bantis, John V. Tsimikas and Stelios Georgiou
% (2011) (see below). Right or left censored data can be accommodated as well as
% covariates. This is a two stage approach. In the case where no
% covariates are present, the Kaplan Meier estimate for the cumulative
% Hazard is derived. In the second stage a monotone spline is fitted to the
% jumps of the Kaplan Meier estimate. If covariates are available, then the
% first stage involves fitting a Cox model, and the spline is fitted to the
% baseline cumulative hazard (baseline is taken at zero).The spline uses the
% truncated power representation. The fitted spline model is
% linear beyond the last knot, and zero before the first knot. The second
% property provides an imitation of the Kaplan Meier estimator, if the first 
% knot is placed at the first event time. However, due to this property,
% caution is needed during the knot placement, especially near the lowest
% values of the "x". Commonly survival data are positive supported but this works for real
% valued data also). 

% INPUT ARGUMENTS: (The first 5 input arguments are obligatory)
%x     :    time to event variable that may be subject to right censoring
%status:    A Boolean array of the same size as x that is 1 for observations
%           that are right-censored and 0 for observations that are observed 
%           exactly.
%Z     :    A matrix that contains all covariates in its columns. The number
%           of its rows should be equal to the size of x. If no covariates are
%           available set as []. In the presence of covariates, a cox model is fitted
%           and smoothing is applied over the baseline cumulative step hazard.
%knots :    The knots of the spline that may be supplied by the user. Alternatively,
%           it can be set to 'auto' and an automatic choice between some knot
%           placement schemes will take over. The selection of the most 
%           appropriate scheme is based on the distance (sum of squares) of the Kaplan Meier's  
%           cumulative step hazard corners (occurring at event times), from the spline model.
%kgon :     A positive integer(>0). An optimal polygon (8*kgon+2) is used
%           to approximate the region of monotonicity. Check the references. If you
%           are unsure about this value just set it equal to 2. 
%           WE STRONGLY RECOMMEND NOT TO USE VERY LARGE VALUES:
%           Computational time will increase dramatically, and probably no
%           better results will be obtained than using k=2 or 3.
%options:   As defined in the built in MATLAB routine coxphfit. Refers only to the 
%           cox model fitting procedure, dealing with maximum number of iterations 
%           as well as the toleration of convergence. See the coxphfit documentation.
%
%
%OPTIONAL INPUT ARGUMENTS: 
%plots:     Can be set as 'cumulative hazard', 'survivor', 'cdf' or 'none' to plot the
%           corresponding functions along with Kaplan Meier based corresponding
%           function. In the case where covariates (Z) are available the
%           baseline corresponding functions are plotted (i.e. at Z=0, for
%           all covariates). The 'none' option does not create any graph
%           and allows you to proceed to the next optional input arguments.
%profil:    An array of length equal to the number of columns of the covariate
%           matrix Z. It may contain a specific profile of covariate values and the
%           estimates produced will refer to that specific profile. If it is
%           set to 0, then estimates for the baseline are obtained. 
%           (The 'plots' input argument must be given if the 'profil' argument
%           is to be used).
%profilplots: Can be set as 'cumulative hazard', 'survivor', 'cdf' to plot the
%             corresponding functions along with the empirical corresponding
%             function for the specific profile given in 'profil'.
%
%
%
%OUTPUT ARGUMENTS: 
%
%   bhat   : the coefficients of the spline model 
%   Hx     : a function handle for the fitted spline for the cumulative hazard
%   Fx     : a function handle for the fitted spline for the cdf
%   Sx     : a function handle for the fitted spline for the survival function  
%   KMdist : the distance of the KM estimator (if you find a knot scheme that 
%            provides a smaller distance that the one provided by the 'auto' just use it! A plot is always useful.)
%   knots  : the knots used.
%   gcoxhat: the coefficients of the ordinary cox model in the case where
%            covariates are present. In the case of no covariates then gcoxhat=NaN.
%   Also additional plots are given if required.
% 
% For the derivation of confidence intervals see file 'HCNSboots'.
%
% Example: (No covariates)
% n=300;
% x=wblrnd(3,3,n,1);
% c=wblrnd(3,3,n,1);
% xcen=min(x,c);
% status=(x>c); %Thus, the available data for analysis are: xcen, status
% 
% [bhat Hx Fx Sx KMdist knots]=HCNS(xcen, status,0, [], 'auto', 2,[], 'survivor')
% 
% A plot of the survival function will pop up along with the KM survival.
% Now one can compute the survival at 2 by typing:
% Sx(2)
% Simirarly the cdf or the cumulative hazard is calculated by:
% Fx(2) or Hx(2) respectively. 

% A covariate matrix can be straightforwardly passed in the third argument.

% For the derivation of confidence intervals the percentile bootstap is
% proposed. Pairs of data of the "x" and "status" are drawn with
% replacement and the spline if fitted to the bootstrap samples. (300 bootstrap samples 
% or more are recommended). A simulation study showed satisfactory
% coverage.

% For examples see the examples.m file.

%Last updated: (25th January 2012).
%Code Author: Leonidas E. Bantis, 
%University of the Aegean, 
%Dept. of Statistics and Actuarial-Financial Mathematics, 
%Samos Island, Greece.

%e-mail: lbantis@aegean.gr, leobantis@gmail.com.
%An updated version of this software will be available online:

%-------www.leobantis.net23.net------------------------------


%References:

%Leonidas E. Bantis, Stelios D. Georgiou and John V. Tsimikas
%(2012). A MATLAB routine for Survival Estimation through the Cumulative Hazard Function with
%Monotone Natural Cubic Splines. Submitted for publication.

%Leonidas E. Bantis, John V. Tsimikas and Stelios D. Georgiou
%(2012). Survival Estimation through the Cumulative Hazard Function with
%Monotone Natural Cubic Splines. (Lifetime Data Analysis (To appear). Springer.)



%[bhat Hx Fx Sx KMdist knots gcoxhat]=HCNS(x, status, Z, knots, kgon, plots, profil, plotsprofil)


% options = statset('Display','final','TolX',toler,'TolFun',toler, 'TolBnd',toler, 'MaxIter', iter, 'MaxFunEvals', iter)
%[b,logL,H,stats] = coxphfit(Zsam,time,'censoring',status,'baseline',0,'options',options);

%---Begin Error Checking:
if nargin<=5;error('The function ''HCNS'' requires at least 5 arguments');end
if nargin>10;error('Too many input arguments in function ''HCNS'' ');end
if isnumeric(x)~=1;error('Time variable must be numeric.');end
%if islogical(status)~=1;error('Status variable must be a boolean array, with 0 for event and 1 for censoring.');end
if max(status)>1;error('status variable must be a boolean (logical) vector, with 0 for an event and 1 for censoring.');end
if min(status)<-1;error('status variable must be a boolean (logical) vector, with 0 for an event and 1 for censoring.');end


sizex=size(x);
if sizex(1)>1 && sizex(2)>1; error('Input argument ''x'' must be a row or a column vector');end
if sizex(1)==1;x=x';end

statusx=size(status);
if statusx(1)>1 && statusx(2)>1; error('Input argument ''status'' must be a row or a column vector');end
if statusx(1)==1;status=status';end

if length(x)~=length(status);error('Time and Status variables must be of same lengths');end

%if length(status)~=length(x);error('Input argument ''status'' and ''x'' must have the same length');end
%if length(find(status==0))+length(find(status==1))~=length(status);error('Input argument ''status'' must values equal to zero or one');end
if length(find(status==1))==length(status);error('No analysis can be performed if all data are censored');end
if length(find(status==-1))==length(status);error('No analysis can be performed if all data are censored');end


if sum(size(defineevent))~=2;error('The ''definevent'' value must be a scalar stating the value that the status variable takes in the case of an event  ');end
if defineevent~=0;error('The ''defineevent'' must be ''0'' ');end


%if nargin>=4 && isempty(Z)~=1
if isnumeric(Z)~=1;error('Not valid argument for covariate matrix. If there are no covariates set the covariate input argument as [].');end

if isempty(Z)~=1
    sizeZ=size(Z);
    if sizeZ(1)~=length(x);error('The dimensions of the covariate matrix ''Z'' does not match the dimension of the time to event variable ''x''  ');end
end


if nargin>=8
    if ischar(plots)~=1;error('The plots argument must be of type char and contain one of the following: ''cumulative hazard'' or ''cdf'' or ''survivor''  ');end;
end

if floor(kgon)~=ceil(kgon);error('The parameter ''k'' must be a positive integer greater than or equal to 1');end
if kgon<1;error('Parameter ''k'', that defines an (8k+2)-gon must be a positive integer, value 2 is recommended');end
if isnumeric(kgon)==0;error('Parameter ''k'', that defines an (8k+2)-gon must be a positive integer, value 2 is recommended');end
if kgon~=round(kgon);error('Parameter ''k'', that defines an (8k+2)-gon must be a positive integer, value 2 is recommended');end



if isnumeric(knots)==1
    sizekn=size(knots);
    if sizekn(1)>1 && sizekn(2)>1;error('Knots must be an array and not a matrix');end
    if sizekn(1)>1;knots=knots';end
end


if nargin>=9
    if isempty(profil)==1;error('Not valid input argument for ''profil''  ');end
    if isnumeric(profil)~=1;error('Input argument for ''profil'' must be numeric ');end
    sizeprof=size(profil);
    if sizeprof(1)>1 && sizeprof(2)>1;error('Input argument for ''profil'' must be an array and not a matrix');end
    
    sizeprof=size(profil); if sizeprof(2)>1;profil=profil';end    
    if length(profil)~=sizeZ(2);('The length of the profile array ''profil'' cannot exceed the number of the covariate matrix Z');end
end

if nargin>=9 && isempty(Z)==1
    error('Since there are no covariates available, no estimates or plots can be obtained for a specific covariate profile');
end

if nargin==10
    if ischar(plotsprofil)~=1;error('The ''plotsprofil'' argument must be of type char and contain one of the following: ''cumulative hazard'' or ''cdf'' or ''survivor''  ');end;
end





    
%---------------Done with error checking----------------------------



xor=x;statusor=status; %original data
if isempty(Z)==1
        gcoxhat=NaN;
        stats=NaN;
        if isnumeric(knots)==1 %&& length(knots)==6
            
            knots=sort(knots);sizekn=size(knots);
            if sizekn(1)>1;knots=knots';end
            if sizekn(1)>1 && sizekn(2)>1;error('Knots must be an array and not a matrix');end
            if min(knots)>=max(xor(statusor==0));error('Reconsider the knot placement: The minimum knot is greater than or equal to the maximum event time. The knots should be scattered among the data.');end
            if max(knots)<=min(xor(statusor==0));error('Reconsider the knot placement: The maximum knot is less than or equal to the minimum event time. The knots should be scattered among the data.');end
           
            %[y x]=ecdf(x,'censoring',status,'function','cumulative hazard');
            [y, x]=ecdfdoubly(x,status);y=1-y;y=-log(y); %S=exp(-H) %log(S)=-H
            x1=x;y1=y;t=knots;
            
             X=zeros(length(x),(length(t)-1)); %preallocate
            
            for i=1:(length(t)-1)
                X(:,i)=(x-t(i)).^3.*(x>t(i));
            end
%             

            options = optimset('LargeScale', 'off','Display', 'off','MaxFunEvals',18000,'MaxIter',18000);
            [A,k,Aeq]=conlsqlin(t,kgon); %find the required constraints using the knots "t".
            [bhat] = lsqlin(X,y,A,(k),Aeq,0,[],[],[],options);

            g=bhat;
            
            % The last parameter can be explicitly calculated by:
           
            g(length(bhat)+1)= (g(1:(end-1))') * (t(1:(end-2))-t(end-1))' / (t(end-1)-t(end));
            %--------------------------------------------
            %DONE WITH ESTIMATING THE SPLINE COEFFICIENTS
            %--------------------------------------------

            t=t';

            Hx=@(x)  Hazx(x,t,g);
            Fx=@(x)  1-exp(-(Hazx(x,t,g)));
            Sx=@(x)  1-Fx(x);

            
            KMdist=sum((Hx(x1)-y1').^2);
            bhat=g;


            knots=t(1:end);
 
 
        %----If the knots are to be automatically selected:

      
        elseif ischar(knots)==1 && strncmp(knots,'auto',4)==1 
               [bhat Hx Fx Sx KMdist knots]=HCNSsupd(xor, statusor,kgon);
            %if the knots aren't a real valued vector or a character vector named 'auto' then return an error:
        else 
            error('Invalid value for ''knots'' was provided.')
        end  % end the "if" command concerning the nature of the "knot" input.
 

%-Check if any plots are requested and provide them in the case where there are no covariates:
    if nargin==8 && isempty(Z)==1
        %error checking
        list={'cumulative hazard','survivor','cdf','none'};
        w=sum(strncmp(plots,list,17));
        
%        w = strmatch(plots, char('cumulative hazard', 'survivor', 'cdf'));
        if w==0;error('Invalid value of ''function'' parameter. You must provide one of the following: ''cumulative hazard'' or ''cdf'' or ''survivor'' ') ; end
        
        gr=linspace(min(min(knots),min(xor)),max(xor(statusor==0)),700);
     
            switch plots
                case 'cumulative hazard'
%                    [y x]=ecdf(xor,'censoring',statusor,'function','cumulative hazard');
                    [y, x]=ecdfdoubly(xor,statusor);y=1-y;y=-log(y); %S=exp(-H) %log(S)=-H
                    plot(gr,Hx(gr'),'r');hold on;stairs(x,y);plot(knots,zeros(1,length(knots)),'or');legend('Spline cumulative hazard','Turnbull cumulative hazard','knots','Location', 'NorthWest');
                    hold off
                case 'survivor'
%                   [y x]=ecdf(xor,'censoring',statusor,'function','survivor');
                    [y, x]=ecdfdoubly(xor,statusor);y=1-y;%y=-log(y); %S=exp(-H) %log(S)=-H
                    plot(gr,Sx(gr),'r');hold on;stairs(x,y);plot(knots,zeros(1,length(knots)),'or');legend('Spline survival function','Turnbull survival','knots','Location', 'NorthEast');
                    hold off
                case 'cdf'
%                   [y x]=ecdf(xor,'censoring',statusor,'function','cdf');
                    [y, x]=ecdfdoubly(xor,statusor);%y=1-y;y=-log(y); %S=exp(-H) %log(S)=-H
                    plot(gr,1-Sx(gr),'r');hold on;stairs(x,y);plot(knots,zeros(1,length(knots)),'or');legend('Spline cdf function','Turnbull cdf','knots','Location', 'NorthWest');
                    hold off
                
            end
    end
    
%-End of plotting in the case of no covariates ----------------------------





%---------------------If there are available covariates:-------------------

elseif isempty(Z)~=1
    
    if isempty(options)==1
    [gcox0,~,Hcox0,stats] = coxphfit(Z,xor,'censoring', statusor,'baseline',0);
    else
    [gcox0,~,Hcox0,stats] = coxphfit(Z,xor,'censoring', statusor,'baseline',0,'options',options);
    end
    
    x1=Hcox0(:,1);H0=Hcox0(:,2);gcoxhat=gcox0;
            if isnumeric(knots)==1 
                if min(knots)>=max(xor(statusor==0));error('Reconsider the knot placement: The minimum knot is greater than or equal to the maximum event time. The knots should be scattered among the data.');end
                if max(knots)<=min(xor(statusor==0));error('Reconsider the knot placement: The maximum knot is less than or equal to the minimum event time. The knots should be scattered among the data.');end
            
                [bhat Hcox Fcox Scox KMdist]=Hcnscox(x1,H0,knots,kgon);
                Hx=Hcox;Fx=Fcox;Sx=Scox;
                
            
            elseif ischar(knots)==1 && strncmp(knots,'auto',4)==1 
                   [bhat Hx Fx Sx KMdist knots]=HCNSsupcox(xor, statusor,Z,kgon);
            else error('Undefined function for input argument knots')   
   

            end
            
            %---------------Plot any graph that is required (in the case where covariates are present)
            %---------------All graphs will refer to the baseline (that is Z=0).
            if nargin>=8
                  %error checking
                  list={'cumulative hazard','survivor','cdf','none'};
                  w=sum(strncmp(plots,list,17));
                  % w = strmatch(plots, char('cumulative hazard', 'survivor', 'cdf'));
                  if w==0;error('Invalid value of ''function'' parameter. You must provide one of the following: ''cumulative hazard'' or ''cdf'' or ''survivor'' ') ; end
    
                  gr=linspace(min(min(knots),min(xor)),max(xor(statusor==0)),700);
                  
                     switch plots
                         case 'cumulative hazard'
                              plot(gr,Hx(gr),'r');hold on;stairs(x1,H0);plot(knots,zeros(1,length(knots)),'or');title('Estimate for the baseline cumulative hazard (Z=0)');legend('Spline cumulative hazard','Cox cumulative hazard','knots','Location', 'NorthWest');
                              hold off
                          case 'survivor'
                              plot(gr,Sx(gr),'r');hold on;stairs(x1,exp(-H0));plot(knots,zeros(1,length(knots)),'or');title('Estimate for the baseline survival (Z=0)');legend('Spline survival function','Cox survival','knots','Location', 'NorthEast');
                              hold off
                          case 'cdf'
                              plot(gr,Fx(gr),'r');hold on;stairs(x1,1-exp(-H0));plot(knots,zeros(1,length(knots)),'or');title('Estimate for the baseline cdf (Z=0)');legend('Spline cdf function','Cox cdf','knots','Location', 'NorthWest');
                              hold off
                     end
            end

            %----If a specific profile for Z is provided by the user:
            
            if nargin>=9
                    linpre=sum(gcoxhat.*profil); %linear predictor of the cox model
                    Hx=@(x) Hx(x).*exp(linpre);
                    Sx=@(x) Sx(x).^(exp(linpre));
                    Fx=@(x) 1-Sx(x);
            
                    %HcoxZ=H0.*exp(linpre);
                    %ScoxZ=exp(-HcoxZ);
                    %FcoxZ=1-ScoxZ;
            
                    if nargin==10
                                %error checking
                                list={'cumulative hazard','survivor','cdf','none'};
                                w=sum(strncmp(plotsprofil,list,17));
                                % w = strmatch(plots, char('cumulative hazard', 'survivor', 'cdf'));
                                if w==0;error('Invalid value of ''function'' parameter. You must provide one of the following: ''cumulative hazard'' or ''cdf'' or ''survivor'' ') ; end
    
                                gr=linspace(min(min(knots),min(xor)),max(xor(statusor==0)),700);
                  
                             switch plotsprofil
                                    case 'cumulative hazard'
                                         figure
                                         plot(gr,Hx(gr),'r');hold on;stairs(x1,H0.*exp(linpre));title('Estimate of the cumulative hazard for the selected profile');legend('Spline cumulative hazard','Cox cumulative hazard','Location', 'NorthWest');
                                         hold off
                                    case 'survivor'
                                         figure                                         
                                         plot(gr,Sx(gr),'r');hold on;stairs(x1,exp(-H0.*exp(linpre)));title('Estimate of the survival for the selected profile');legend('Spline survival function','Cox survival','Location', 'NorthEast');
                                         hold off
                                    case 'cdf'
                                         figure                                         
                                         plot(gr,Fx(gr),'r');hold on;stairs(x1,1-exp(-H0.*exp(linpre)));title('Estimate of the cdf for the selected profile');legend('Spline cdf function','Cox cdf','Location', 'NorthWest');
                                         hold off
                             end

                    end %end nargin==10
                    
            end %end nargin>=9

                
end %end function
            
            








end %end function


%----Function to evaluate the cumulative hazard 

function out=Hazx(x,t,g)
%Needs the point at which you wish to evaluate the H(x) ("x")
%Needs the knots "t"
%And the estimated coefs of the spline "g"
sizex=size(x);if sizex(1)==1;x=x';end
Hazx = @(x) sum(bsxfun(@times,g(:),bsxfun(@minus,x',t(:)).^3 .*bsxfun(@gt,x',t(:))),1);
out=Hazx(x);
   if isnan(out)==1;out=Inf;end         
end


