function [rochat, AUChat, rocnaive, AUCnaive, exitflag]=roclod(xcen,ycen,statusx,statusy,method,init)
% Derives a smooth ROC estimate for a biomarker subject to a limit of
% detection, either a lower or an upper.
%INPUT ARGUMENTS:
%xcen: a column with the scores of the healthy.
%ycen: a column with the scores of the diseased.
%statusx: refers to xcen and it is a column taking the value -1 for a left censored measurement due a lower LOD, 0
%for an exactly observed measurement, and 1 for a right censored
%measurement due to an upper LOD.
%statusx: refers to ycen and it is a column taking the value -1 for a left censored measurement due a lower LOD, 0
%for an exactly observed measurement, and 1 for a right censored
%measurement due to an upper LOD.
%method: can only be one the following: 'BC', 'GG', 'Kernel(BC)',
%'Kernel(GG)', 'HCNS'. 'BC' refers to the power normal fit a.k.a. Box-Cox,
%'GG' refers to the generalized gamma based ROC, 'Kernel(BC)' refers to the
%kernels for which the power normal fit has been used to perform multiple
%imputation to the hidden tails of each distribution. An average of 30
%kernel smoothed ROC estimates in then produced to be the final estimate.
%Kernel(GG): same as Kernel(BC) but the GG is considered for the multiple
%imputation. HCNS: refers to the spline method as described in the paper
%with automatic knot selection. 
%init: initial values that apply only for the BC and Kernel(BC) methods. If
%this argument is not reached then the means and std of the complete data
%are taken for initial values of the four parameters of the two groups. For
%the lambda 1 is the default initial value. This argument can either not be
%reached at all or it can be set to []. If provided it should contain a row vector of 5 values: 
%a guess for mx, sx, my, sy and lambda with that order. For the GG and Kernel(GG) methods
%initial values are based on the fit of a Weibull. The HCNS does not
%require initial values.
%OUTPUT ARGUMENTS:
%rochat: a function handle that refers to the selected by the user
%methodology.
%AUChat: the area under the rochat.
%rocnaive: a function handle that refers to the Naive Empirical ROC.
%AUCnaive: the area under rocnaive.
%exitflag: s scalar that is equal to 1 if the selected method has converged
%and zero otherwise.
%EXAMPLE: see "example_of_software" file for a reproducible example in cell mode.
% Author: 
%Leonidas E. Bantis,
%Dept. Biostatistics,
%The University of Texas MD Anderson Cancer Center.
%Any use of this software implies reference to the relevant paper:
%"Estimation of smooth ROC curves for biomarkers with limits of detection".
%==========================







xcen1=xcen;nx=length(xcen);
ycen1=ycen;ny=length(ycen);

if length(find(statusx==0))==0
    error('there must be at least some exactly observed measurements for the healthy group')
end

if length(find(statusy==0))==0
    error('there must be at least some exactly observed measurements for the diseased group')
end


if min(statusx)<-1 || max(statusx)>1
    error('statusx must be either -1 for left censoring, either 0 for exactly observed, or 1 for right censoring')
end


if min(statusy)<-1 || max(statusy)>1
    error('statusy must be either -1 for left censoring, either 0 for exactly observed, or 1 for right censoring')
end


cutlow=min(min(xcen(statusx==-1)),min(ycen(statusy==-1)));
cutupp=max(max(xcen(statusx==1)),max(ycen(statusy==1)));

if nargin<6 
    init=[mean(xcen1(statusx==0))   std(xcen1(statusx==0))  mean(ycen1(statusy==0))   std(ycen1(statusy==0))   1];
elseif nargin==6 && isempty(nargin)==1
    init=[mean(xcen1(statusx==0))   std(xcen1(statusx==0))  mean(ycen1(statusy==0))   std(ycen1(statusy==0))   1];
else
    init=init;
end
    
%===========NAIVE EMPIRICAL==================
scores=[xcen;ycen];
labels=[zeros(nx,1);ones(ny,1)];

[X Y T AUC]=perfcurve(labels, scores, 1);AUCnaive=AUC;
plot(X,Y)
hold on
rocemp=@(t) roct(t,X,Y);
Xnai=X;Ynai=Y;
%==============EMPIRICAL=============


switch method
    case 'BC'
        whichmethod=1;

%==============BOX-COX===============
if nargin>5
    [g2, roctbc, exitflag, gval]=boxcoxcens(xcen1,ycen1,statusx,statusy,init);
else
    [g2, roctbc, exitflag, gval]=boxcoxcens(xcen1,ycen1,statusx,statusy,init);
end

gr=0:0.01:1;
% hold on
% plot(gr, roctbc(gr),'g','LineWidth',1.5)
% xlabel('FPR');ylabel('TPR')
rochat=roctbc;
%====================================

    case 'GG'

whichmethod=2;
%========================================================
%===============GENERALIZED GAMMA========================
%========================================================

%========GENERALIZED GAMMA=============

%Use the broader model of the generalized model
%Z=[ones(length(xcen1),1) data(:,3)];
%[pars covpars SE CI Zscores pvalues gval exitflag]=aft(xcen1,[Z],statusx,'gamma',[],1,[],1);

Z=[ones(length(xcen1),1) xcen1.*0];
[pars covpars SE CI Zscores pvalues gval exitflag1]=aft(xcen1,[Z],statusx,'gamma',[],1,[],1);

d=pars(end);s=pars(end-1);inter=pars(1);
a=d/s;g=1/d^2;lam=exp(-inter);

Sa=@(x)  (a<0).*(gammainc(g.*(lam.*x).^a, g ))+...
           (a>0).*((gammainc(g.*(lam.*x).^a, g,'upper')));

       
Sinva=@(u)  (a<0).*((gammaincinv(u, g )./g).^(1./a))./lam+...
          (a>0).*((gammaincinv(u, g,'upper')./g).^(1./a))./lam;

SE1=sum(SE);
Z=[ones(length(ycen1),1) ycen1.*0];
[pars covpars SE CI Zscores pvalues gval exitflag2]=aft(ycen1,[Z],statusy,'gamma',[],1,[],1);

d=pars(end);s=pars(end-1);inter=pars(1);
a=d/s;g=1/d^2;lam=exp(-inter);

Sb=@(x)  (a<0).*(gammainc(g.*(lam.*x).^a, g ))+...
           (a>0).*((gammainc(g.*(lam.*x).^a, g,'upper')));

       
Sinvb=@(u)  (a<0).*((gammaincinv(u, g )./g).^(1./a))./lam+...
          (a>0).*((gammaincinv(u, g,'upper')./g).^(1./a))./lam;


SE2=sum(SE);
    
%roctgg=@(t) Sb(Sinva(1-t))
t=0:0.01:1;
rocegg=@(t) Sb(Sinva(t));

%plot(t,roctgg(t,Sa,Sb),'c','LineWidth',2)
%hold on
%plot(t,rocegg(t),'y','LineWidth',2)
rochat=rocegg;

exitflag=1;
if exitflag1~=1 || exitflag2~=1
    exitflag=0;
end

    case 'kernelBC'
whichmethod=3;
%======================================================================
%===============================KERNELBC===============================
if isempty(cutlow)==1;cutlow=-Inf;end
if isempty(cutupp)==1;cutupp=+Inf;end    

[kernrocBC exitflag]=rocLODkernBC(xcen,ycen,statusx, statusy, cutlow, cutupp,init);
rochat=kernrocBC;

    case 'kernelGG'
        whichmethod=4;

%======================================================================
%===============================KERNELGG===============================


[kernrocGG exitflag]=rocLODkernGG(xcen,ycen,statusx, statusy, cutlow, cutupp);
gr=0:0.01:1;
hold on
%figure
%plot(gr, kernrocGG(gr),'r','LineWidth',1.5)
rochat=kernrocGG;
    case 'HCNS'
        whichmethod=5;
%=====================================================================
%============================HCNS=====================================
mygrid=0.001:0.01:0.999;

exitflag=1;

%==============Spline===============
reverseit=0;

minn=min(min(xcen),min(ycen));
time=xcen;
status=statusx;
if min(statusx)~=-1
[bhat Hx Fx Sx KMdist knots gcoxhat]=HCNS(time, status, 0, [], 'auto',2,[]  );
%Estimation procedure is now over.
elseif min(statusx)==-1 && max(statusx)==1
[bhat Hx Fx Sx KMdist knots gcoxhat]=HCNSd(time, status, 0, [], 'auto',2,[]);
%Estimation procedure is now over.
elseif min(statusx)==-1 && max(statusx)==0
    flag1=1;
[bhat Hx Fx Sx KMdist knots gcoxhat]=HCNS(-time+minn+10, abs(status), 0, [], 'auto',2,[]);reverseit=1;
%[bhat Hx Fx Sx KMdist knots gcoxhat]=HCNSd(time, status, 0, [], 'auto',2,[]);

%Estimation procedure is now over.
end
flag2=0;
hold on
time=ycen;
status=statusy;

if min(statusy)~=-1
[bhat Hy Fy Sy KMdist knots gcoxhat]=HCNS(time, status, 0, [], 'auto',2,[]  );
%Estimation procedure is now over.
elseif min(statusy)==-1 && max(statusy)==1
[bhat Hy Fy Sy KMdist knots gcoxhat]=HCNSd(time, status, 0, [], 'auto',2,[]);
%Estimation procedure is now over.
elseif min(statusy)==-1 && max(statusy)==0 
    flag2=1;
[bhat Hy Fy Sy KMdist knots gcoxhat]=HCNS(-time+minn+10, abs(status), 0, [], 'auto',2,[]);reverseit=1;
%[bhat Hy Fy Sy KMdist knots gcoxhat]=HCNSd(time, status, 0, [], 'auto',2,[]);
%Estimation procedure is now over.
end



if flag1==1 || flag2==1
    [bhat Hx Fx Sx KMdist knots gcoxhat]=HCNS(-xcen+minn+10, abs(statusx), 0, [], 'auto',2,[]);reverseit=1;
    [bhat Hy Fy Sy KMdist knots gcoxhat]=HCNS(-ycen+minn+10, abs(statusy), 0, [], 'auto',2,[]);reverseit=1;
end


u=mygrid;
gr=0.001:0.001:max(max(xcen),max(ycen))+300;
%roccns=@(t) (1-Fy(Finvrcs(Fx,gr,1-t))) ;
%plot(u, roccns(u),'g')
%roctbc=roccns;
%if reverseit==0
%roctbc=@(u) ROCCNS(Fx, Fy, [u], xcen, ycen);
%else
%roctbc=@(u) ROCCNS(Fx, Fy, [1-u], ycen, xcen); %reversed
%end



if flag1==1 || flag2==1
    [bhat Hx Fx Sx KMdist knots gcoxhat]=HCNS(-xcen+minn+10, abs(statusx), 0, [], 'auto',2,[]);reverseit=1;
    [bhat Hy Fy Sy KMdist knots gcoxhat]=HCNS(-ycen+minn+10, abs(statusy), 0, [], 'auto',2,[]);reverseit=1;
end


if (flag1+flag2)==0
   roctbc=@(u) ROCCNS(Fx, Fy, [u], xcen, ycen);
else
   gr=(min(min(-xcen),min(-ycen))-300):0.001:max(max(xcen),max(ycen))+300;
   %roctbc=@(u) ROCCNS(Fy, Fx, [u], xcen, ycen);
   roctbc=@(u) 1-ROCCNS(Fx, Fy, [1-u], xcen, ycen);
end

roctHCNS=roctbc;gr=0:0.01:1;
%plot(gr,roctHCNS(gr),'c')
rochat=roctHCNS;


end

%==========================================================

%close all
rochat=@(t) rochat(t).*(t<1)+1.*(t==1);
rochat=@(t) rochat(t).*(t>0)+0;

rochat=@(t) rochat(t);
t=0:0.01:1;
plot(t,rochat(t),'r')
hold on
rocnaive=rocemp;
plot(Xnai,Ynai);

AUChat=quadgk(@(t) rochat(t),0,1);

if whichmethod==1;legend('Proposed BC ROC','Naive Empirical','Location','SouthEast');end
if whichmethod==2;legend('Proposed GG ROC','Naive Empirical','Location','SouthEast');end
if whichmethod==3;legend('Proposed Kernel(BC) ROC','Naive Empirical','Location','SouthEast');end
if whichmethod==4;legend('Proposed Kernel(GG) ROC','Naive Empirical','Location','SouthEast');end
if whichmethod==5;legend('Proposed HCNS(spline) ROC','Naive Empirical','Location','SouthEast');end

end